'''Predator functions for use in optimizer'''
# from adapting import *
# from energy_cluster import *
# from fingerprint_niche import *
# from fitpred_bests import *
# from fitpred_new import *
# from fitpred import *
# from mutation_dups_energy import *
# from mutation_dups import *
# from mutation_dups_zp import *
# from mutation_dups_quench import *