'''General fitness functions for use in optimizer'''
# from chempotswap import *
# from enpafit import *
# from enthalpyfit import *
# from exponentialfit import *
# from formationenergy import *
# from stem_cost import *
# from surfaceenergy import *
# from totalenfit import *
# from stem_cost_rotation import *
# from sibias import *
# from sibias2 import *