############
NTYPES
DISP
DOSIN
DOSEND
DOSSMEAR
DOSSTEP
DXSTART
EIGSIZE
IPRINT
LCENTRAL
LDRIFT
LEIGEN
LFORCEOUT
LFREE
LGAMMA
LNOSYM
LRECIP
LSUPER
LSYMM
MASS
NAME
NCYCLESEIG
ND
NDIM
NTI
PTEMP
QA
QB
QC
QI
QF
SYMPREC
TEMPERATURE
USETHIS
############
