'''General switch functions for use in optimizer'''
from crossover_switch import *
from moves_switch import *
#from predator_switch import *
from selection_switch import *
#from lambdacommamu import *
from fitness_switch import *