'''General functions for selection'''
# from best import *
# from cost import *
# from fuss import *
# from fuss1 import *
# from fuss1r import *
# from fuss2 import *
# from fussf import *
# from fussr import *
# from metropolis import *
# from multitournament import *
# from random_pick import *
# from rank import *
# from tournament import *
# from tournament1 import *
# from tournament2 import *
