'''Functions for use in optimizer'''
#import crossover
#import fingerprinting
#import fitness
#import generate
#import inp_out
#import moves
#import predator
#import selection
#import switches
#import tools
#from Optimizer import Optimizer
